# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against issue 72

<https://github.com/heuer/segno/issues/72>
"""
import pytest
from segno.helpers import _make_epc_qr_data as make_epc_qr_data, make_epc_qr
from .test_helpers_epcqr import _make_valid_kw


@pytest.mark.parametrize('encoding, number', (('uTf-8', 1),
                                              ('utf-8', 1),
                                              ('iso-8859-1', 2),
                                              ('ISO-8859-2', 3),
                                              ('iSo-8859-4', 4),
                                              ('iso-8859-5', 5),
                                              ('iso-8859-7', 6),
                                              ('iso-8859-10', 7),
                                              ('iso-8859-15', 8)))
def test_valid_encoding(encoding, number):
    kw = _make_valid_kw()
    kw['name'] = 'Simple name'
    kw['encoding'] = encoding
    d = make_epc_qr_data(**kw).split(b'\n')
    assert str(number).encode() == d[2]
    qr = make_epc_qr(**kw)
    assert qr


@pytest.mark.parametrize('encoding', ('utf-16', 'iso-8859-6', 'ascii',
                                      'something'))
def test_illegal_encoding(encoding):
    kw = _make_valid_kw()
    kw['encoding'] = encoding
    with pytest.raises(ValueError) as ex:
        make_epc_qr_data(**kw)
    assert 'encoding' in str(ex.value)
    with pytest.raises(ValueError) as ex:
        make_epc_qr(**kw)
    assert 'encoding' in str(ex.value)


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Test against issue #3.
<https://github.com/heuer/segno/issues/3>
"""
from __future__ import unicode_literals, absolute_import
import io
import segno
try:
    from .tutils import matrix_looks_valid
# Attempted relative import in non-package
except (ValueError, SystemError, ImportError):
    from tutils import matrix_looks_valid


def test_issue_3():
    qr = segno.make_micro('test')
    assert 'M3' == qr.version
    # This fails since PNG operates with a fixed set of two colors
    qr.save(io.BytesIO(), kind='png')


def test_issue_3_autodetect_micro():
    qr = segno.make('test')
    assert 'M3' == qr.version
    # This fails since PNG operates with a fixed set of two colors
    qr.save(io.BytesIO(), kind='png')


def test_issue_3_matrix():
    qr = segno.make_micro('test')
    is_ok, msg = matrix_looks_valid(qr.matrix)
    assert is_ok, msg


if __name__ == '__main__':
    import pytest
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
PNG related tests.
"""
from __future__ import unicode_literals, absolute_import
import io
import pytest
import segno
from segno import writers as colors
from png import Reader as PNGReader


def test_greyscale():
    qr = segno.make_qr('test')
    buff = io.BytesIO()
    qr.save(buff, kind='png', quiet_zone='white')
    buff.seek(0)
    reader = PNGReader(file=buff)
    reader.preamble()
    assert reader.greyscale


def test_not_greyscale():
    qr = segno.make_qr('test')
    buff = io.BytesIO()
    qr.save(buff, kind='png', quiet_zone=None)  # Three "colors": white, black, transparent
    buff.seek(0)
    reader = PNGReader(file=buff)
    reader.preamble()
    assert not reader.greyscale
    palette = reader.palette()
    assert 3 == len(palette)
    assert 0 == palette[0][3]  # Transparent color
    assert (0, 0, 0, 255) in palette  # black
    assert (255, 255, 255, 255) in palette  # white


def test_plte():
    qr = segno.make_qr('test')
    buff = io.BytesIO()
    dark = (0, 0, 139)
    qr.save(buff, kind='png', dark=dark, light=None)
    buff.seek(0)
    reader = PNGReader(file=buff)
    reader.preamble()
    assert not reader.greyscale
    palette = reader.palette()
    assert 2 == len(palette)
    assert 0 == palette[0][3]  # Transparent color
    dark_with_alpha = dark + (255,)
    assert dark_with_alpha in palette


def test_plte_no_transparency():
    qr = segno.make_qr('test')
    buff = io.BytesIO()
    dark = (0, 0, 139)
    light = (255, 255, 255)
    qr.save(buff, kind='png', dark=dark, light=light)
    buff.seek(0)
    reader = PNGReader(file=buff)
    reader.preamble()
    assert not reader.greyscale
    palette = reader.palette()
    assert 2 == len(palette)
    assert dark in palette
    assert light in palette


def test_plte_colors():
    qr = segno.make_qr('test')
    buff = io.BytesIO()
    dark = (0, 0, 139)
    light = (255, 255, 255)
    qr.save(buff, kind='png', dark=dark, light=light, quiet_zone='green',
            finder_dark='purple', finder_light='yellow')
    buff.seek(0)
    reader = PNGReader(file=buff)
    reader.preamble()
    assert not reader.greyscale
    palette = reader.palette()
    assert 5 == len(palette)
    assert dark in palette
    assert light in palette
    assert colors._color_to_rgb('green') in palette
    assert colors._color_to_rgb('purple') in palette
    assert colors._color_to_rgb('yellow') in palette


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against the ``writers`` module.
"""
from __future__ import absolute_import, unicode_literals
import os
import io
import tempfile
import pytest
from segno import consts, writers


def test_writable_stream():
    buff = io.BytesIO()
    with writers.writable(buff, 'wb') as f:
        f.write(b'x')
    assert not buff.closed


def test_writable_stream2():
    buff = io.StringIO()
    with writers.writable(buff, 'wt') as f:
        f.write('x')
    assert not buff.closed


def test_writable_stream3():
    buff = io.StringIO()
    with pytest.raises(Exception):
        with writers.writable(buff, 'wt') as f:
            f.write('x')
            raise Exception()
    assert not f.closed


def test_writable_not_stream():
    fn = tempfile.NamedTemporaryFile()
    name = fn.name
    fn.close()
    try:
        with writers.writable(name, 'wb') as f:
            assert name == f.name
            f.write(b'Segno')
    finally:
        os.remove(name)


def test_writable_not_stream2():
    fn = tempfile.NamedTemporaryFile()
    name = fn.name
    fn.close()
    try:
        with writers.writable(name, 'wt') as f:
            assert name == f.name
            f.write('Segno')
    finally:
        os.remove(name)


def test_writable_not_stream3():
    fn = tempfile.NamedTemporaryFile()
    name = fn.name
    fn.close()
    with pytest.raises(Exception):
        with writers.writable(name, 'wb') as f:
            assert name == f.name
            f.write(b'Segno')
            raise Exception()
    assert f.closed


def test_colormap_dark_light():
    cm = writers._make_colormap(7, dark='blue', light='white')
    assert 15 == len(cm)


def test_colormap_lesser_version_7():
    cm = writers._make_colormap(6, dark='blue', light='white')
    assert 13 == len(cm)
    assert consts.TYPE_VERSION_DARK not in cm
    assert consts.TYPE_VERSION_LIGHT not in cm


def test_colormap_micro():
    cm = writers._make_colormap(consts.VERSION_M3, dark='blue', light='white')
    assert 10 == len(cm)
    assert consts.TYPE_VERSION_DARK not in cm
    assert consts.TYPE_VERSION_LIGHT not in cm
    assert consts.TYPE_ALIGNMENT_PATTERN_DARK not in cm
    assert consts.TYPE_ALIGNMENT_PATTERN_LIGHT not in cm
    assert consts.TYPE_DARKMODULE not in cm


if __name__ == '__main__':
    pytest.main([__file__])
